@@ -333,10 +333,16 @@ notebook_build_children (GladeXML *xml, GtkWidget *w, GladeWidgetInfo *info,
 		if (tmp2 == NULL || strcmp(attr->value, "Notebook:tab") != 0)
 			pages = g_list_append (pages, child);
 		else {
-			GtkWidget *page = pages->data;
+			GtkWidget *page;
 			gint key = glade_xml_get_parent_accel(xml);
 
-			pages = g_list_remove (pages, page);
+			if (pages) {
+				page = pages->data;
+				pages = g_list_remove (pages, page);
+			} else {
+				page = gtk_label_new("Unknown page");
+				gtk_widget_show(page);
+			}
 			gtk_notebook_append_page (GTK_NOTEBOOK(w), page,child);
 			if (key) {
 				gtk_widget_add_accelerator(page, "grab_focus",
@@ -525,12 +531,18 @@ toolbar_build_children (GladeXML *xml, GtkWidget *w, GladeWidgetInfo *info,
 			}
 			if (icon) {
 				GdkPixmap *pix;
-				GdkBitmap *mask;
+				GdkBitmap *mask = NULL;
 				pix = gdk_pixmap_colormap_create_from_xpm(NULL,
 					gtk_widget_get_colormap(w), &mask,
 					NULL, icon);
 				g_free(icon);
-				iconw = gtk_pixmap_new(pix, mask);
+				if (pix)
+					iconw = gtk_pixmap_new(pix, mask);
+				else
+					iconw = gtk_type_new(
+						gtk_pixmap_get_type());
+				if (pix) gdk_pixmap_unref(pix);
+				if (mask) gdk_bitmap_unref(mask);
 			}
 			if (!strcmp(cinfo->class, "GtkToggleButton"))
 				child = gtk_toolbar_append_element(
@@ -654,6 +666,38 @@ fontselectiondialog_build_children (GladeXML *xml, GtkWidget *w,
 	}
 }
 
+static void
+inputdialog_build_children (GladeXML *xml, GtkWidget *w,
+			    GladeWidgetInfo *info,const char *longname)
+{
+	GList *tmp;
+
+	for (tmp = info->children; tmp; tmp = tmp->next) {
+		GladeWidgetInfo *cinfo = tmp->data;
+		GtkWidget *child = NULL;
+		GList *tmp2;
+		gchar *child_name = NULL;
+
+		for (tmp2 = cinfo->attributes; tmp2; tmp2 = tmp2->next) {
+			GladeAttribute *attr = tmp2->data;
+			if (!strcmp(attr->name, "child_name")) {
+				child_name = attr->value;
+				break;
+			}
+		}
+		if (!child_name) continue;
+
+		if (!strcmp(child_name, "InputDialog:save_button"))
+			child = GTK_INPUT_DIALOG(w)->save_button;
+		else if (!strcmp(child_name, "InputDialog:close_button"))
+			child = GTK_INPUT_DIALOG(w)->close_button;
+
+		if (child)
+			glade_xml_set_common_params(xml, child, cinfo,
+						    longname);
+	}
+}
+
 static void
 combo_build_children (GladeXML *xml, GtkWidget *w,
 		      GladeWidgetInfo *info, const char *longname)
@@ -1575,7 +1619,8 @@ pixmap_new(GladeXML *xml, GladeWidgetInfo *info)
 {
 	GtkWidget *pix;
 	GList *tmp;
-	GdkPixmap *pixmap; GdkBitmap *bitmap;
+	GdkPixmap *pixmap;
+	GdkBitmap *bitmap = NULL;
 	char *filename = NULL;
   
 	for (tmp = info->attributes; tmp; tmp = tmp->next) {
@@ -1595,6 +1640,9 @@ pixmap_new(GladeXML *xml, GladeWidgetInfo *info)
 		pix = gtk_pixmap_new(pixmap, bitmap);
 	else
 		pix = gtk_type_new(gtk_pixmap_get_type());
+
+	if (pixmap) gdk_pixmap_unref(pixmap);
+	if (bitmap) gdk_bitmap_unref(bitmap);
 	misc_set(GTK_MISC(pix), info);
 
 	return pix;
@@ -2349,6 +2397,37 @@ preview_new(GladeXML *xml, GladeWidgetInfo *info)
 	return preview;
 }
 
+static GtkWidget *
+calendar_new (GladeXML *xml, GladeWidgetInfo *info)
+{
+	GtkWidget *wid = gtk_calendar_new();
+	GList *tmp;
+	GtkCalendarDisplayOptions dopt = 0;
+
+	for (tmp = info->attributes; tmp; tmp = tmp->next) {
+		GladeAttribute *attr = tmp->data;
+
+		if (!strcmp(attr->name, "show_heading")) {
+			if (attr->value[0] == 'T')
+				dopt |= GTK_CALENDAR_SHOW_HEADING;
+		} else if (!strcmp(attr->name, "show_day_names")) {
+			if (attr->value[0] == 'T')
+				dopt |= GTK_CALENDAR_SHOW_DAY_NAMES;
+		} else if (!strcmp(attr->name, "no_month_change")) {
+			if (attr->value[0] == 'T')
+				dopt |= GTK_CALENDAR_NO_MONTH_CHANGE;
+		} else if (!strcmp(attr->name, "show_week_numbers")) {
+			if (attr->value[0] == 'T')
+				dopt |= GTK_CALENDAR_SHOW_WEEK_NUMBERS;
+		} else if (!strcmp(attr->name, "week_start_monday")) {
+			if (attr->value[0] == 'T')
+				dopt |= GTK_CALENDAR_WEEK_START_MONDAY;
+		}
+	}
+	gtk_calendar_display_options(GTK_CALENDAR(wid), dopt);
+	return wid;
+}
+
 static GtkWidget *
 window_new (GladeXML *xml, GladeWidgetInfo *info)
 {
@@ -2496,6 +2575,24 @@ fontselectiondialog_new (GladeXML *xml, GladeWidgetInfo *info)
 	return win;
 }
 
+static GtkWidget *
+inputdialog_new (GladeXML *xml, GladeWidgetInfo *info)
+{
+	GtkWidget *win = gtk_input_dialog_new();
+	GList *tmp;
+
+	for (tmp = info->attributes; tmp; tmp = tmp->next) {
+		GladeAttribute *attr = tmp->data;
+
+		if (!strcmp(attr->name, "title"))
+			gtk_window_set_title(GTK_WINDOW(win), _(attr->value));
+	}
+	glade_xml_set_window_props(GTK_WINDOW(win), info);
+	glade_xml_set_toplevel(xml, GTK_WINDOW(win));
+
+	return win;
+}
+
 static GtkWidget *
 custom_new (GladeXML *xml, GladeWidgetInfo *info)
 {
@@ -2602,6 +2699,7 @@ static const GladeWidgetBuildData widget_data[] = {
 	{"GtkColorSelection",colorselection_new,NULL},
 	{"GtkFontSelection", fontselection_new, NULL},
 	{"GtkPreview",       preview_new,       NULL},
+	{"GtkCalendar",      calendar_new,      NULL},
 
   /* toplevel widgets */
 	{"GtkWindow",        window_new,        glade_standard_build_children},
@@ -2611,6 +2709,7 @@ static const GladeWidgetBuildData widget_data[] = {
 				    colorselectiondialog_build_children},
 	{"GtkFontSelectionDialog", fontselectiondialog_new,
 				   fontselectiondialog_build_children},
+	{"GtkInputDialog",   inputdialog_new,   inputdialog_build_children},
 
   /* the custom widget */
 	{"Custom",           custom_new,        NULL},
